import json
from pathlib import Path
from typing import Any

import click

CONFIG_DIR = Path.home() / ".config" / "cli"
CONFIG_FILE = CONFIG_DIR / "config.json"
DEFAULT_CONFIG_DIR = Path.home() / ".config" / "cli"


class Config:
    @staticmethod
    def path():
        return "config_path"

    @staticmethod
    def _default_config():
        return {
            Config.path(): str(DEFAULT_CONFIG_DIR),
        }

    @staticmethod
    def load(config_path: Path = CONFIG_FILE):
        if not config_path.exists():
            return Config._default_config()
        with open(config_path, "r") as file:
            return json.load(file)

    @staticmethod
    def save(config: dict[str, Any]):
        if not CONFIG_DIR.exists():
            CONFIG_DIR.mkdir(parents=True, exist_ok=True)
        with open(CONFIG_FILE, "w") as file:
            json.dump(config, file, indent=4)


@click.group()
def config():
    """Configuration options"""
    pass


@config.command()
@click.argument("pair")
@click.pass_context
def create(ctx: click.Context, pair: str):
    """Set a configuration key-value pair."""
    try:
        key, value = pair.split("=", 1)
        click.secho(f"Key: {key}", fg="green")
        click.secho(f"Value: {value}", fg="blue")
        config = ctx.obj.get("config", {})
        config[key] = value
        ctx.obj["config"] = config
        Config.save(config)
    except ValueError:
        click.secho("Error: Please provide input in the format 'key=value'.", fg="red")

@config.command()
@click.argument("key", required=False)
@click.pass_context
def read(ctx: click.Context, key: str):
    """Read a configuration value by key or print all key-value pairs."""
    config = ctx.obj.get("config", {})

    if key is None:
        # Print all key-value pairs
        if config:
            for k, v in config.items():
                click.secho(f"{k}={v}", fg="green")
        else:
            click.secho("No config data found.", fg="red")
    else:
        # Read the value for the specified key
        value = config.get(key)
        if value is not None:
            click.secho(f"{key}={value}", fg="green")
        else:
            click.secho(f"Error: Key '{key}' not found in config.", fg="red")

@config.command()
@click.argument("key")
@click.pass_context
def remove(ctx: click.Context, key: str):
    """Remove a configuration value by key."""
    config = ctx.obj.get("config", {})
    value = config.get(key)
    if value is not None:
        click.secho(f"Removed {key} from config", fg="red")
        del config[key]
        ctx.obj["config"] = config
        Config.save(config)
    else:
        click.secho(f"Error: Key '{key}' not found in config.", fg="red")



"""
A simple example of chatting to an LLM with Textual.

Lots of room for improvement here.

See https://textual.textualize.io/blog/2024/09/15/anatomy-of-a-textual-user-interface/

"""
import os.path
import threading
from pathlib import Path

import click
# /// script
# requires-python = ">=3.12"
# dependencies = [
#     "llm",
#     "textual",
# ]
# ///
from textual import on, work
from textual.app import App, ComposeResult
from textual.containers import VerticalScroll
from textual.widgets import Footer, Header, Input, Markdown

from utils.file_util import FileUtil

try:
    from ollama import chat
except ImportError:
    raise ImportError(
        'install the ollama package or run with "python -m pip install ollama"'
    ) from None
# The system prompt
SYSTEM = """Formulate all responses as if you where the sentient AI named Cyborg from the Alien movies."""


class Prompt(Markdown):
    """Markdown for the user prompt."""
    BORDER_TITLE = "Me"


class Response(Markdown):
    """Markdown for the reply from the LLM."""

    BORDER_TITLE = "Cyborg"


def stream_string(long_text):
    """
    Simulates streaming a long string by yielding chunks of it.

    Args:
        long_text: The long string to be streamed.

    Yields:
        Chunks of the long_text.
    """
    chunk_size = 50  # Adjust chunk size as needed
    for i in range(0, len(long_text), chunk_size):
        yield long_text[i:i + chunk_size]


class CyborgApp(App):
    """Simple app to demonstrate chatting to an LLM."""

    BINDINGS = [
        ("ctrl+c", "stop_prompt", "Stop Query"),  # Define Ctrl+C binding
        ('space', 'focus_input', 'Focus Input'),
    ]
    AUTO_FOCUS = "Input"

    CSS = """
    Prompt {
        border: wide $primary;
        background: $primary 10%;
        color: $text;
        margin: 1;        
        margin-left: 8;
        padding: 1 2 0 2;
    }

    Response {
        border: wide $success;
        background: $success 10%;   
        color: $text;             
        margin: 1;      
        margin-right: 8; 
        padding: 1 2 0 2;
    }

     .error-response {
        background: red !important;  /* Red background for stop event */
        color: white !important;      /* White text when stopped */
    }
    """
    stop_query = False
    chat_history_file = os.path.join("assets", "chat.json")

    def __init__(self):
        super().__init__()
        self.stop_event = threading.Event()
        FileUtil.create_if_not_exists(self.chat_history_file)
        cached_history = FileUtil.read_json_file(self.chat_history_file)
        self.history = [] if (cached_history is None or 'chat' not in cached_history) else cached_history['chat']

    def compose(self) -> ComposeResult:
        yield Header()
        with VerticalScroll(id="chat-view"):
            yield Response("INTERFACE 2037 READY FOR INQUIRY")
        yield Input(placeholder="How can I help you?")
        yield Footer()

    async def on_mount(self) -> None:
        chat_view = self.query_one("#chat-view")
        for message in self.history:
            if message['role'] == 'user':
                await chat_view.mount(Prompt(message['content']))
            else:
                await chat_view.mount(Response(message['content']))
        chat_view.scroll_end()  # This ensures the view scrolls to the bottom


    @on(Input.Submitted)
    async def on_input(self, event: Input.Submitted) -> None:
        """When the user hits return."""
        chat_view = self.query_one("#chat-view")
        event.input.clear()
        await chat_view.mount(Prompt(event.value))
        await chat_view.mount(response := Response())
        response.anchor()

        self.query_one(Input).blur()
        self.stop_event.clear()
        self.send_prompt(event.value, response)

    @work(thread=True)
    def send_prompt(self, prompt: str, response: Response) -> None:
        """Get the response in a thread."""

        self.history.append({
            'role': 'user',
            'content': prompt,
        })

        response_content = ""
        try:
            llm_response = chat(model='llama3.2', messages=self.history, stream=True)
            for chunk in llm_response:
                if self.stop_event.is_set():  # Check the stop event
                    response_content += "\n\n[b]!!Prompt Stopped!!![/b]"
                    # Dynamically change the background color to red
                    self.call_from_thread(response.update, response_content)
                    self.call_from_thread(response.add_class, "error-response")
                    break
                if 'content' in chunk['message']:
                    response_content += chunk['message']['content']
                    self.call_from_thread(response.update, response_content)
            if response_content:
                self.history.append({
                    'role': 'assistant',
                    'content': response_content,
                })
        except Exception as e:
            self.notify(f"Oops! Something went wrong: {e}", severity="error", timeout=10)
        finally:
            self.action_focus_input()
            chat_history = FileUtil.read_json_file(self.chat_history_file)
            chat_history['chat'] = self.history
            FileUtil.write_json_file(self.chat_history_file, chat_history)

    def action_stop_prompt(self):
        """Handles Ctrl+C to stop the prompt"""
        self.stop_event.set()  # Set the stop event
        self.notify("Ctrl+C Pressed. Stopping the prompt", severity="error", timeout=3)

    def action_focus_input(self):
        self.query_one(Input).focus()

@click.group()
def cyborg():
    """A cli friendly cyborg tool"""
    pass

@cyborg.command()
@click.pass_context
def run(ctx: click.Context):
    click.secho("Running CLI Cyborg", fg="green")
    CyborgApp().run()


if __name__ == "__main__":
    app = CyborgApp()
    app.run()



import click

from utils.config import Config, config as config_group
from bot.cyborg import cyborg

@click.group()
@click.version_option()
@click.pass_context
def cli(ctx: click.Context):
    """Main CLI group."""
    # Initialize ctx.obj as a dictionary if not already set
    if ctx.obj is None:
        ctx.obj = {}
    local_config = Config.load()
    ctx.obj["config"] = local_config


# Register the `config` commands
cli.add_command(config_group)
cli.add_command(cyborg)

if __name__ == "__main__":
    cli()



import json
import os

from pathlib import Path

class FileUtil:
    encoding = 'utf-8'

    @staticmethod
    def save(path: str, content: str, file_type=None):
        try:
            file_path = Path(path)
            if file_type is None:
                file_path.write_text(content, encoding=FileUtil.encoding)
            elif file_type == 'json':
                with open(path, 'w') as f:
                    json.dump(content, f, indent=2)
            else:
                raise TypeError('file_type must be "json" or None')
        except Exception as e:
            print(f"An error occurred saving {path}: {e}")


    @staticmethod
    def read(path: str, file_type: str=None, graceful=True):
        file_path = Path(path)
        if not file_path.exists():
            if graceful:
                return None
            raise FileNotFoundError(f"File {path} not found.")
        if file_type is None:
            return file_path.read_text(encoding=FileUtil.encoding)
        elif file_type is 'json':
            return json.loads(file_path.read_text(encoding=FileUtil.encoding))
        else:
            raise NotImplementedError(f'invalid file type - {file_type}')

    @staticmethod
    def create_if_not_exists(path):
        """
          Creates the file at the specified path if it doesn't exist.
          Also creates the directory if it doesn't exist.
          """
        if not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        if not os.path.exists(path):
            FileUtil.write_json_file(path, {})

    @staticmethod
    def read_json_file(file_path):
        try:
            with open(file_path, 'r') as f:
                return json.load(f)
        except FileNotFoundError as ex:
            print("File not found")
            raise ex
        except json.JSONDecodeError as ex:
            print("Invalid JSON Format")
            raise ex

    @staticmethod
    def write_json_file(file_path, data):
        try:
            with open(file_path, 'w') as f:
                json.dump(data, f, indent=2)
        except Exception as ex:
            print(f"An error occurred: {ex}")


